using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Discounts;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CouponRepositoryNoCache : ICouponRepository
    {
        #region Private

        private const string AllFields = " CouponID, DiscountType, DiscountAmount, Percentage, " +
                "ExpirationType, ExpirationDate, ExpirationQuantity, CurrentQuantity, MerchantNotes, " +
                "ProductFilter, ProductIDs, CategoryIDs, CustomerFilter, CustomerUserName, " +
                "MinimumSubtotal, ProductCostType, RepeatDiscount, MinimumQuantity, PromotionQuantity, FreeShippingType ";

        private Coupon CreateCoupon( Coupon coupon )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO Coupon (CouponID, DiscountType, DiscountAmount, Percentage, " +
                "ExpirationType, ExpirationDate, ExpirationQuantity, CurrentQuantity, MerchantNotes, " +
                "ProductFilter, ProductIDs, CategoryIDs, CustomerFilter, CustomerUserName, " +
                "MinimumSubtotal, ProductCostType, RepeatDiscount, MinimumQuantity, PromotionQuantity, FreeShippingType " +
                ") " +
                "VALUES (@CouponID, @DiscountType, @DiscountAmount, @Percentage, " +
                "@ExpirationType, @ExpirationDate, @ExpirationQuantity, @CurrentQuantity, @MerchantNotes, " +
                "@ProductFilter, @ProductIDs, @CategoryIDs, @CustomerFilter, @CustomerUserName, " +
                "@MinimumSubtotal, @ProductCostType, @RepeatDiscount, @MinimumQuantity, @PromotionQuantity, @FreeShippingType " +
                "); ",
                DataAccess.CreateParameterString( coupon.CouponID ),
                DataAccess.CreateParameterString( coupon.DiscountType.ToString() ),
                DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( coupon.DiscountAmount ) ),
                DataAccess.CreateParameterDouble( Convert.ToDouble( coupon.Percentage ) ),
                DataAccess.CreateParameterString( coupon.ExpirationType.ToString() ),
                DataAccess.CreateParameterDateTime( coupon.ExpirationDate ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.ExpirationQuantity ) ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.CurrentQuantity ) ),
                DataAccess.CreateParameterString( coupon.MerchantNotes ),
                DataAccess.CreateParameterString( coupon.ProductFilter.ToString() ),
                DataAccess.CreateParameterString( coupon.ProductIDs ),
                DataAccess.CreateParameterString( coupon.CategoryIDs ),
                DataAccess.CreateParameterString( coupon.CustomerFilter.ToString() ),
                DataAccess.CreateParameterString( coupon.CustomerUserName ),
                DataAccess.CreateParameterDecimal( coupon.MinimumSubtotal ),
                DataAccess.CreateParameterString( coupon.ProductCostType.ToString() ),
                DataAccess.CreateParameterBool( coupon.RepeatDiscount ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.MinimumQuantity ) ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.PromotionQuantity ) ),
                DataAccess.CreateParameterString( coupon.FreeShippingType.ToString() ) );

            return coupon;
        }

        private Coupon UpdateCoupon( Coupon coupon, string newCouponID )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Coupon " +
                "SET CouponID = @NewCouponID, DiscountType = @DiscountType, DiscountAmount = @DiscountAmount, " +
                "Percentage = @Percentage, ExpirationType = @ExpirationType, " +
                "ExpirationDate = @ExpirationDate, ExpirationQuantity = @ExpirationQuantity, " +
                "CurrentQuantity = @CurrentQuantity, MerchantNotes = @MerchantNotes, " +
                "ProductFilter = @ProductFilter, ProductIDs = @ProductIDs, " +
                "CategoryIDs = @CategoryIDs, CustomerFilter = @CustomerFilter, " +
                "CustomerUserName = @CustomerUserName, MinimumSubtotal = @MinimumSubtotal, ProductCostType = @ProductCostType,  " +
                "RepeatDiscount = @RepeatDiscount, MinimumQuantity = @MinimumQuantity, PromotionQuantity = @PromotionQuantity, " +
                "FreeShippingType = @FreeShippingType " +
                "WHERE CouponID = @CouponID ;",
                DataAccess.CreateParameterString( newCouponID ),
                DataAccess.CreateParameterString( coupon.DiscountType.ToString() ),
                DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( coupon.DiscountAmount ) ),
                DataAccess.CreateParameterDouble( Convert.ToDouble( coupon.Percentage ) ),
                DataAccess.CreateParameterString( coupon.ExpirationType.ToString() ),
                DataAccess.CreateParameterDateTime( coupon.ExpirationDate ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.ExpirationQuantity ) ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.CurrentQuantity ) ),
                DataAccess.CreateParameterString( coupon.MerchantNotes ),
                DataAccess.CreateParameterString( coupon.ProductFilter.ToString() ),
                DataAccess.CreateParameterString( coupon.ProductIDs ),
                DataAccess.CreateParameterString( coupon.CategoryIDs ),
                DataAccess.CreateParameterString( coupon.CustomerFilter.ToString() ),
                DataAccess.CreateParameterString( coupon.CustomerUserName ),
                DataAccess.CreateParameterDecimal( coupon.MinimumSubtotal ),
                DataAccess.CreateParameterString( coupon.ProductCostType.ToString() ),
                DataAccess.CreateParameterBool( coupon.RepeatDiscount ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.MinimumQuantity ) ),
                DataAccess.CreateParameterInt32( Convert.ToInt32( coupon.PromotionQuantity ) ),
                DataAccess.CreateParameterString( coupon.FreeShippingType.ToString() ),
                DataAccess.CreateParameterString( coupon.CouponID ) );

            coupon.CouponID = newCouponID;
            return coupon;
        }

        private Coupon.FreeShippingTypeEnum GetFreeShippingType( string freeShippingType )
        {
            if (String.IsNullOrEmpty( freeShippingType ))
                return Coupon.FreeShippingTypeEnum.MatchingItemsOnly;
            else
                return (Coupon.FreeShippingTypeEnum) Enum.Parse( typeof( Coupon.FreeShippingTypeEnum ), freeShippingType );
        }

        private Coupon.DiscountTypeEnum GetDiscountType( string discountType )
        {
            if (String.IsNullOrEmpty( discountType ))
                return Coupon.DiscountTypeEnum.Price;
            else
                return (Coupon.DiscountTypeEnum) Enum.Parse( typeof( Coupon.DiscountTypeEnum ), discountType );
        }

        private Coupon.ExpirationTypeEnum GetExpirationType( string expirationType )
        {
            if (String.IsNullOrEmpty( expirationType ))
                return Coupon.ExpirationTypeEnum.Quantity;
            else
                return (Coupon.ExpirationTypeEnum) Enum.Parse( typeof( Coupon.ExpirationTypeEnum ), expirationType );
        }

        private Coupon.ProductFilterEnum GetProductFilter( string productFilter )
        {
            if (String.IsNullOrEmpty( productFilter ))
                return Coupon.ProductFilterEnum.All;
            else
                return (Coupon.ProductFilterEnum) Enum.Parse( typeof( Coupon.ProductFilterEnum ), productFilter );
        }

        private Coupon.CustomerFilterEnum GetCustomerFilter( string customerFilter )
        {
            if (String.IsNullOrEmpty( customerFilter ))
                return Coupon.CustomerFilterEnum.All;
            else
                return (Coupon.CustomerFilterEnum) Enum.Parse( typeof( Coupon.CustomerFilterEnum ), customerFilter );
        }

        private Coupon.ProductCostTypeEnum GetProductCostType( string productCostType )
        {
            if (String.IsNullOrEmpty( productCostType ) ||
                productCostType == Coupon.ProductCostTypeEnum.AnyProduct.ToString())
                return Coupon.ProductCostTypeEnum.AnyProduct;
            else
                return Coupon.ProductCostTypeEnum.CouponEligibleProducts;
        }

        private Coupon CreateOneFromRow( DataRow row )
        {
            Coupon coupon = new Coupon();
            coupon.CouponID = row["CouponID"].ToString();
            coupon.DiscountType = GetDiscountType( row["DiscountType"].ToString() );
            coupon.DiscountAmount = ConvertUtilities.ToDecimal( row["DiscountAmount"] );
            coupon.Percentage = ConvertUtilities.ToDouble( row["Percentage"] );
            coupon.ExpirationType = GetExpirationType( row["ExpirationType"].ToString() );
            coupon.ExpirationDate = ConvertUtilities.ToDateTime( row["ExpirationDate"] );
            coupon.ExpirationQuantity = ConvertUtilities.ToInt32( row["ExpirationQuantity"] );
            coupon.CurrentQuantity = ConvertUtilities.ToInt32( row["CurrentQuantity"] );
            coupon.MerchantNotes = row["MerchantNotes"].ToString();
            coupon.ProductFilter = GetProductFilter( row["ProductFilter"].ToString() );
            coupon.ProductIDs = row["ProductIDs"].ToString();
            coupon.CategoryIDs = row["CategoryIDs"].ToString();
            coupon.CustomerFilter = GetCustomerFilter( row["CustomerFilter"].ToString() );
            coupon.CustomerUserName = row["CustomerUserName"].ToString();
            coupon.MinimumSubtotal = ConvertUtilities.ToDecimal( row["MinimumSubtotal"] );
            coupon.ProductCostType = GetProductCostType( row["ProductCostType"].ToString() );
            coupon.RepeatDiscount = ConvertUtilities.ToBoolean( row["RepeatDiscount"] );
            coupon.MinimumQuantity = ConvertUtilities.ToInt32( row["MinimumQuantity"] );
            coupon.PromotionQuantity = ConvertUtilities.ToInt32( row["PromotionQuantity"] );
            coupon.FreeShippingType = GetFreeShippingType( row["FreeShippingType"].ToString() );

            return coupon;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string CreateAmountTypeWithAnd( string fieldName )
        {
            if (String.Compare( fieldName, "DiscountAmount", true ) == 0)
                return "AND DiscountType = 'Price' ";
            else if (String.Compare( fieldName, "Percentage", true ) == 0)
                return "AND DiscountType = 'Percentage' ";
            else
                return String.Empty;
        }

        private string CreateExpirationTypeWithAnd( string fieldName )
        {
            if (String.Compare( fieldName, "ExpirationDate", true ) == 0)
                return "AND ( ExpirationType = 'Date' OR ExpirationType = 'Both' ) ";
            else if (String.Compare( fieldName, "ExpirationQuantity", true ) == 0)
                return "AND ( ExpirationType = 'Quantity' OR ExpirationType = 'Both' ) ";
            else
                return String.Empty;
        }

        private IList<Coupon> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Coupon> list = new List<Coupon>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        #region Public Methods

        public Coupon Create( Coupon coupon )
        {
            return CreateCoupon( coupon );
        }

        public Coupon Update( Coupon coupon )
        {
            return UpdateCoupon( coupon, coupon.CouponID );
        }

        public Coupon Update( Coupon coupon, string newCouponID )
        {
            return UpdateCoupon( coupon, newCouponID );
        }

        public Coupon GetOne( string couponID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CouponID, DiscountType, DiscountAmount, Percentage, " +
                "ExpirationType, ExpirationDate, ExpirationQuantity, CurrentQuantity, MerchantNotes, " +
                "ProductFilter, ProductIDs, CategoryIDs, CustomerFilter, CustomerUserName, " +
                "MinimumSubtotal, ProductCostType, RepeatDiscount, MinimumQuantity, PromotionQuantity, FreeShippingType " +
                "FROM Coupon " +
                "WHERE CouponID = @CouponID; ",
                DataAccess.CreateParameterString( couponID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Coupon.Null;
        }

        public void Delete( string couponID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Coupon " +
                "WHERE CouponID = @Coupon ;",
                DataAccess.CreateParameterString( couponID ) );
        }

        public void DeleteAll()
        {
            DataAccess.ExecuteNonQuery( "DELETE FROM Coupon;" );
        }

        public IList<Coupon> SearchCoupon(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT CouponID, DiscountType, DiscountAmount, " +
                "    Percentage, ExpirationType, " +
                "    ExpirationDate, ExpirationQuantity, " +
                "    CurrentQuantity, MerchantNotes, " +
                "    ProductFilter, ProductIDs, " +
                "    CategoryIDs, CustomerFilter, " +
                "    CustomerUserName, MinimumSubtotal, ProductCostType, " +
                "    RepeatDiscount, MinimumQuantity, PromotionQuantity, FreeShippingType " +
                "FROM Coupon " +
                commandSearch +
                CreateAmountTypeWithAnd( searchFilter.FieldName ) +
                CreateExpirationTypeWithAnd( searchFilter.FieldName ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["CouponID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CouponID, DiscountType, DiscountAmount, " +
                "    Percentage, ExpirationType, " +
                "    ExpirationDate, ExpirationQuantity, " +
                "    CurrentQuantity, MerchantNotes, ProductCostType, " +
                "    RepeatDiscount, MinimumQuantity, PromotionQuantity, FreeShippingType " +
                "FROM Coupon " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
