﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Shared.DataAccess;

namespace Vevo.Data.Implementation
{
    internal class ProductDepartmentAccess
    {
        #region Private

        private IList<String> ConvertToDepartmentIDs( DataTable table )
        {
            IList<String> list = new List<String>();
            foreach (DataRow row in table.Rows)
            {
                list.Add( row["DepartmentID"].ToString() );
            }

            return list;
        }

        #endregion


        #region Public Methods

        public void SaveAll( string productID, IList<String> DepartmentIDs )
        {
            DeleteByProductID( productID );
            Create( productID, DepartmentIDs );
        }

        public void Create( string productID, IList<String> DepartmentIDs )
        {
            foreach (string departmentID in DepartmentIDs)
            {
                int result = DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductDepartment (ProductID, DepartmentID) " +
                    "VALUES (@ProductID, @DepartmentID); ",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( departmentID ) );

                if (result != 1)
                {
                    Exception exception = new ApplicationException(
                            "Failed creating product. " +
                        "Inserting into ProductDepartment returns " + result.ToString() );
                    DataAccessHelper.LogError( exception );
                    throw exception;
                }
            }
        }

        public void DeleteByProductID( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ProductDepartment " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );
        }

        public IList<String> GetByProductID( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT DepartmentID " +
                "FROM ProductDepartment " +
                "WHERE ProductID = @ProductID; ",
                DataAccess.CreateParameterString( productID ) );

            return ConvertToDepartmentIDs( table );
        }

        #endregion

    }
}
