using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class StateRepository : IStateRepository
    {
        #region Private

        private const string RepositoryName = "StateRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetAllByCountryCodeSignature = "GetAllByCountryCode";
        private const string TableName = "State";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private StateRepositoryNoCache _repository = new StateRepositoryNoCache();

        #endregion

        #region Constructors

        public StateRepository()
        {
        }

        public StateRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion


        #region Public Methods

        public State Create( State state )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            State newState = _repository.Create( state );
            _dataCache.ClearRepositoryCache();
            return newState;
        }

        public State Update( State state )
        {
            State result = _repository.Update( state );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public State Update( State state, string newStateCode )
        {
            State result = _repository.Update( state, newStateCode );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public State GetOne( string countryCode, string stateCode )
        {
            State result = (State) _dataCache.GetValue( GetOneSignature, _cacheDependency, countryCode, stateCode );

            if (result == null)
            {
                result = _repository.GetOne( countryCode, stateCode );
                _dataCache.SetValue( GetOneSignature, result, countryCode, stateCode );
            }

            return result;
        }

        public void Delete( string countryCode, string stateCode )
        {
            _repository.Delete( countryCode, stateCode );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByCountry( string countryCode )
        {
            _repository.DeleteByCountry( countryCode );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _dataCache.ClearRepositoryCache();
        }

        public IList<State> GetAllByCountryCode( string countryCode, string sortBy, BoolFilter enabled )
        {
            IList<State> list = (IList<State>) _dataCache.GetValue(
                    GetAllByCountryCodeSignature,
                    _cacheDependency,
                    countryCode,
                    sortBy,
                    enabled.ToString()
                    );

            if (list == null)
            {
                list = _repository.GetAllByCountryCode( countryCode, sortBy, enabled );
                _dataCache.SetValue(
                    GetAllByCountryCodeSignature,
                    list,
                    countryCode,
                    sortBy,
                    enabled.ToString()
                    );
            }

            return list;
        }

        #endregion
    }
}
