using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Configurations
{
    public class ApplicationSettings
    {
        #region Private
        private IFileManager _fileManager;
        private bool _installCompleted;
        private bool _clearShoppingCartWhenLogout;

        private bool GetBoolValue( XmlDocument xmlDocument, string settingName )
        {
            XmlNodeList nodeList = xmlDocument.GetElementsByTagName( settingName );
            if (nodeList.Count > 0)
                return ConvertUtilities.ToBoolean( nodeList[0].InnerText.Trim() );
            else
                return false;
        }

        private void LoadSettings()
        {
            string xml = _fileManager.ReadToEnd( SystemConst.ApplicationSettingsFilePath );

            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.LoadXml( xml );

            _installCompleted = GetBoolValue( xmlDocument, "InstallCompleted" );
            _clearShoppingCartWhenLogout = GetBoolValue( xmlDocument, "ClearShoppingCartWhenLogout" );
        }
        #endregion

        #region Constructors
        public ApplicationSettings( IFileManager fileManager )
        {
            _fileManager = fileManager;

            LoadSettings();
        }
        #endregion



        #region Public Properties
        public bool InstallCompleted
        {
            get { return _installCompleted; }
            set { _installCompleted = value; }
        }

        public bool ClearShoppingCartWhenLogout
        {
            get { return _clearShoppingCartWhenLogout; }
            set { _clearShoppingCartWhenLogout = value; }
        }
        #endregion

        #region Public Methods
        public void Save()
        {
            string data = String.Format(
                @"<?xml version=""1.0""?>
                <Settings>
                    <InstallCompleted>{0}</InstallCompleted>
                    <ClearShoppingCartWhenLogout>{1}</ClearShoppingCartWhenLogout>
                </Settings>",
                _installCompleted, _clearShoppingCartWhenLogout );

            _fileManager.SaveAll( SystemConst.ApplicationSettingsFilePath, data );
        }
        #endregion
    }
}