using System.Collections.Generic;
using System.IO;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using System.Web.Security;
using Vevo.Base.Domain;

namespace Vevo.Domain.ImportExport
{
    public class CustomerExporter
    {
        private string GenerateHeader()
        {
            return "\"RegisterDate\",\"UserName\",\"FirstName\",\"LastName\",\"Company\",\"Address1\"," +
                    "\"Address2\",\"City\",\"State\",\"Zip\",\"Country\",\"Phone\",\"Fax\",\"Email\"," +
                    "\"UseBillingAsShipping\",\"ShippingFirstName\",\"ShippingLastName\",\"ShippingCompany\",\"ShippingAddress1\"," +
                    "\"ShippingAddress2\",\"ShippingCity\",\"ShippingState\",\"ShippingZip\",\"ShippingCountry\",\"ShippingPhone\"," +
                    "\"ShippingFax\",\"ShippingResidential\",\"MerchantNotes\",\"IsWholesale\",\"WholesaleLevel\",\"AffiliateCode\"," +
                    "\"AffiliateExpireDate\",\"IsEnabled\"," +
                    "\"IsTaxExempt\",\"QBListID\",\"IsQBExported\",\"ProductIDsCompareList\",\"TaxExemptID\",\"TaxExemptCountry\",\"TaxExemptState\",\"FBUserID\"";
        }

        private string GenerateShippingAddressHeader()
        {
            return "\"UserName\",\"AliasName\",\"ShippingFirstName\",\"ShippingLastName\",\"ShippingCompany\",\"ShippingAddress1\",\"ShippingAddress2\"," +
                   "\"ShippingCity\",\"ShippingState\",\"ShippingZip\",\"ShippingCountry\",\"ShippingPhone\",\"ShippingFax\",\"ShippingResidential\",\"IsSameAsBillingAddress\"";
        }

        private string GenerateDetail( Customer customer )
        {
            string registerDate, userName, firstName, lastName, company, address1, address2;
            string city, state, zip, country, phone, fax, email, userBillingAsShipping, shippingFirstName, shippingLastName;
            string shippingCompany, shippingAddress1, shippingAddress2, shippingCity, shippingState, shippingZip, shippingCountry;
            string shippingPhone, shippingFax, shippingResidential, merchantNotes, isWholesale, wholesaleLevel, affiliateCode;
            string affiliateExpireDate, isEnabled;
            string isTaxExempt, qbListID, isQBExported, productIDsCompareList, taxExemptID, taxExemptCountry, taxExemptState, fbUserID;

            registerDate = ConvertUtilities.ToString( customer.RegisterDate );
            userName = customer.UserName;
            firstName = customer.BillingAddress.FirstName;
            lastName = customer.BillingAddress.LastName;
            company = customer.BillingAddress.Company;
            address1 = customer.BillingAddress.Address1;
            address2 = customer.BillingAddress.Address2;
            city = customer.BillingAddress.City;
            state = customer.BillingAddress.State;
            zip = customer.BillingAddress.Zip;
            country = customer.BillingAddress.Country;
            phone = customer.BillingAddress.Phone;
            fax = customer.BillingAddress.Fax;
            email = customer.Email;
            userBillingAsShipping = ConvertUtilities.ToString( customer.UseBillingAsShipping );
            shippingFirstName = customer.ShippingAddress.FirstName;
            shippingLastName = customer.ShippingAddress.LastName;
            shippingCompany = customer.ShippingAddress.Company;
            shippingAddress1 = customer.ShippingAddress.Address1;
            shippingAddress2 = customer.ShippingAddress.Address2;
            shippingCity = customer.ShippingAddress.City;
            shippingState = customer.ShippingAddress.State;
            shippingZip = customer.ShippingAddress.Zip;
            shippingCountry = customer.ShippingAddress.Country;
            shippingPhone = customer.ShippingAddress.Phone;
            shippingFax = customer.ShippingAddress.Fax;
            shippingResidential = ConvertUtilities.ToString( customer.ShippingAddress.Residential );
            merchantNotes = customer.MerchantNotes;
            isWholesale = ConvertUtilities.ToString( customer.IsWholesale );
            wholesaleLevel = ConvertUtilities.ToString( customer.WholesaleLevel );
            affiliateCode = customer.AffiliateCode;
            affiliateExpireDate = ConvertUtilities.ToString( customer.AffiliateExpireDate );
            isEnabled = ConvertUtilities.ToString( customer.IsEnabled );
            isTaxExempt = ConvertUtilities.ToString( customer.IsTaxExempt );
            qbListID = customer.QBListID;
            isQBExported = ConvertUtilities.ToString( customer.IsQBExported );
            productIDsCompareList = customer.ProductIDsCompareList;
            taxExemptID = customer.TaxExemptID;
            taxExemptCountry = customer.TaxExemptCountry;
            taxExemptState = customer.TaxExemptState;
            fbUserID = customer.FBUserID;

            return registerDate + "," + userName + "," + firstName + "," + lastName + "," + company + "," +
                address1 + "," + address2 + "," + city + "," + state + "," + zip + "," + country + "," + phone + "," + fax + "," +
                email + "," + userBillingAsShipping + "," + shippingFirstName + "," + shippingLastName + "," + shippingCompany + "," +
                shippingAddress1 + "," + shippingAddress2 + "," + shippingCity + "," + shippingState + "," + shippingZip + "," +
                shippingCountry + "," + shippingPhone + "," + shippingFax + "," + shippingResidential + "," + merchantNotes + "," +
                isWholesale + "," + wholesaleLevel + "," + affiliateCode + "," + affiliateExpireDate + "," + isEnabled + "," +
                isTaxExempt + "," + qbListID + "," + isQBExported + "," + productIDsCompareList + "," + taxExemptID + "," +
                taxExemptCountry + "," + taxExemptState + "," + fbUserID;
        }

        private string GenerateShippingAddressDetail( ShippingAddress address )
        {
            string userName, aliasName, shippingFirstName, shippingLastName, shippingCompany, shippingAddress1;
            string shippingAddress2, shippingCity, shippingState, shippingZip, shippingCountry, shippingPhone;
            string shippingFax, shippingResidential, isSameAsBillingAddress;

            userName = DataAccessContext.CustomerRepository.GetOne( address.CustomerID ).UserName;
            aliasName = address.AliasName;
            shippingFirstName = address.FirstName;
            shippingLastName = address.LastName;
            shippingCompany = address.Company;
            shippingAddress1 = address.Address1;
            shippingAddress2 = address.Address2;
            shippingCity = address.City;
            shippingState = address.State;
            shippingZip = address.Zip;
            shippingCountry = address.Country;
            shippingPhone = address.Phone;
            shippingFax = address.Fax;
            shippingResidential = ConvertUtilities.ToString( address.Residential );
            isSameAsBillingAddress = ConvertUtilities.ToString( address.IsSameAsBillingAddress );

            return userName + "," + aliasName + "," + shippingFirstName + "," + shippingLastName + "," + shippingCompany + "," +
                shippingAddress1 + "," + shippingAddress2 + "," + shippingCity + "," + shippingState + "," +
                shippingZip + "," + shippingCountry + "," + shippingPhone + "," + shippingFax + "," + shippingResidential + "," + isSameAsBillingAddress;
        }

        public MemoryStream ExportToStream( IList<Customer> customersList )
        {
            //Create the return memorystream object that will hold the buffered data.
            MemoryStream returnStream = new MemoryStream();

            //Create a streamwriter to write to the memory stream
            StreamWriter sw = new StreamWriter( returnStream );

            if (customersList.Count > 0)
            {
                sw.WriteLine( GenerateHeader() );
                string details;
                foreach (Customer customer in customersList)
                {
                    details = GenerateDetail( customer );
                    sw.WriteLine( details );
                }
            }


            //Clean up the stream writer
            sw.Flush();
            //sw.Close();

            return returnStream;
        }

        public MemoryStream ExportShippingAddressToStream( IList<ShippingAddress> addressList )
        {
            //Create the return memorystream object that will hold the buffered data.
            MemoryStream returnStream = new MemoryStream();

            //Create a streamwriter to write to the memory stream
            StreamWriter sw = new StreamWriter( returnStream );

            if (addressList.Count > 0)
            {
                sw.WriteLine( GenerateShippingAddressHeader() );
                string details;
                foreach (ShippingAddress address in addressList)
                {
                    details = GenerateShippingAddressDetail( address );
                    sw.WriteLine( details );
                }
            }


            //Clean up the stream writer
            sw.Flush();
            //sw.Close();

            return returnStream;
        }

    }
}
