using System;
using System.Data;
using System.Configuration;

using Vevo.Domain.Products;

namespace Vevo.Domain.Orders
{
    /// <summary>
    /// Summary description for OptionItemValue
    /// </summary>
    [Serializable]
    public class OptionItemValue
    {
        #region Private

        private OptionItem _optionItem;
        private OptionGroup.OptionGroupType _groupType;
        private string _details;
        private bool _useStock;
        private TempUploadedFile _tempUploadedFile;


        private void Initialize(
            OptionItem optionItem,
            OptionGroup.OptionGroupType groupType,
            string details,
            bool useStock )
        {
            _optionItem = optionItem;
            _groupType = groupType;
            _details = details;
            _useStock = useStock;
        }

        #endregion


        #region Constructors

        public OptionItemValue(
            OptionItem optionItem,
            OptionGroup.OptionGroupType groupType,
            string details,
            bool useStock )
        {
            Initialize( optionItem, groupType, details, useStock );
        }

        public OptionItemValue(
            OptionItem optionItem,
            OptionGroup.OptionGroupType groupType,
            string details,
            bool useStock,
            TempUploadedFile tempUploadedFile )
        {
            Initialize( optionItem, groupType, details, useStock );

            _tempUploadedFile = tempUploadedFile;
        }

        #endregion


        #region Public Properties

        public OptionItem OptionItem
        {
            get { return _optionItem; }
        }

        public OptionGroup.OptionGroupType GroupType
        {
            get { return _groupType; }
        }

        public string OptionItemID
        {
            get { return _optionItem.OptionItemID; }
        }

        public string Name
        {
            get { return OptionItem.Name; }
        }

        public string Details
        {
            get { return _details; }
        }

        public bool UseStock
        {
            get { return _useStock; }
        }

        public TempUploadedFile TempUploadedFile
        {
            get { return _tempUploadedFile; }
        }

        #endregion


        #region Public Methods

        public string GetDisplayedName(
            Culture culture,
            Currency currency )
        {
            string itemName = OptionItem.Locales[culture].Name;
            string optionText = "";

            if (GroupType == OptionGroup.OptionGroupType.Text ||
                GroupType == OptionGroup.OptionGroupType.Upload ||
                GroupType == OptionGroup.OptionGroupType.UploadRequired)
            {
                if (String.IsNullOrEmpty( Details ))
                    optionText = "N/A";
                else
                    optionText = Details;

                itemName = optionText;
            }

            OptionGroup optionGroup = DataAccessContext.OptionGroupRepository.GetOne(
                culture, OptionItem.OptionGroupID );

            if (OptionItem.OptionItemType == OptionItem.PriceType.Price)
            {
                if (OptionItem.PriceToAdd != 0.00m)
                {
                    return String.Format( "{0} : {1} (cost +{2})",
                        optionGroup.Name,
                        itemName,
                        currency.FormatPrice( OptionItem.PriceToAdd ) );
                }
                else
                {
                    return String.Format( "{0} : {1}",
                        optionGroup.Name,
                        itemName );
                }
            }
            else
            {
                if (OptionItem.PercentageChange != 0)
                {
                    return String.Format( "{0} : {1} (cost +{2}%)",
                        optionGroup.Name,
                        itemName,
                        OptionItem.PercentageChange );
                }
                else
                {
                    return String.Format( "{0} : {1}",
                        optionGroup.Name,
                        itemName );
                }
            }
        }

        #endregion

    }
}