﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Vevo.WebServices.Base;
using Vevo.WebServices.Security;
using Vevo.WebServices.SearchFilters;
using Vevo.WebServices.Customers;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.Domain.DataInterfaces;
using Vevo.WebAppLib;
using Vevo.Domain.Orders;
using Vevo.eBay;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain;

namespace Vevo.WebServices.Orders
{
    public class OrderService : IOrderService
    {
        private OrderConstraints ConvertOrderToConstraints( Order order )
        {
            OrderConstraints constraints = new OrderConstraints();
            constraints.AvsAddrStatus = order.AvsAddrStatus;
            constraints.AvsZipStatus = order.AvsZipStatus;
            constraints.BaseCurrencyCode = order.BaseCurrencyCode;
            constraints.Cancelled = order.Cancelled;
            constraints.ContainsRecurring = order.ContainsRecurring;
            constraints.CouponDiscount = order.CouponDiscount;
            constraints.CouponID = order.CouponID;
            constraints.CustomerComments = order.CustomerComments;
            constraints.CustomerID = order.CustomerID;
            constraints.CvvStatus = order.CvvStatus;
            constraints.Email = order.Email;
            constraints.GatewayOrderID = order.GatewayOrderID;
            constraints.GatewayPaymentStatus = order.GatewayPaymentStatus;
            constraints.GiftCertificate = order.GiftCertificate;
            constraints.GiftCertificateCode = order.GiftCertificateCode;
            constraints.GiftRegistryID = order.GiftRegistryID;
            constraints.HandlingFee = order.HandlingFee;
            constraints.InvoiceNotes = order.InvoiceNotes;
            constraints.IPAddress = order.IPAddress;
            constraints.IsQBExported = order.IsQBExported;
            constraints.OrderDate = order.OrderDate;
            constraints.OrderID = order.OrderID;
           
            constraints.ParentID = order.ParentID;
            constraints.PaymentComplete = order.PaymentComplete;
            constraints.PaymentMethod = order.PaymentMethod;
            constraints.PaymentToken = order.PaymentToken;
            constraints.Processed = order.Processed;
            constraints.QbPostDate = order.QbPostDate;
            constraints.QbReferenceCode = order.QbReferenceCode;
            constraints.QBTxnID = order.QBTxnID;
            constraints.ShippedDate = order.ShippedDate;
            constraints.ShippingCost = order.ShippingCost;
            constraints.ShippingID = order.ShippingID;
            constraints.ShippingMethod = order.ShippingMethod;
            constraints.ShowShippingAddress = order.ShowShippingAddress;
            constraints.Status = order.Status;
            constraints.StoreID = order.StoreID;
            constraints.Subtotal = order.Subtotal;
            constraints.Tax = order.Tax;
            constraints.TrackingMethod = order.TrackingMethod;
            constraints.TrackingNumber = order.TrackingNumber;
            constraints.UserConversionRate = order.UserConversionRate;
            constraints.UserCurrencyCode = order.UserCurrencyCode;
            constraints.UserName = order.UserName;
            constraints.PONumber = order.PONumber;
            constraints.IsTaxExempt = order.IsTaxExempt;
            constraints.TaxExemptID = order.TaxExemptID;
            constraints.TaxExemptCountry = order.TaxExemptCountry;
            constraints.TaxExemptState = order.TaxExemptState;

            constraints.RedeemPoint = order.RedeemPoint;
            constraints.RedeemPrice = order.RedeemPrice;
            constraints.PointEarned = order.PointEarned;
            constraints.IsPointApplied = order.IsPointApplied;
            constraints.IsSubscriptionApplied = order.IsSubscriptionApplied;

            constraints.Billing = new AddressConstraints();
            constraints.Billing.Address1 = order.Billing.Address1;
            constraints.Billing.Address2 = order.Billing.Address2;
            constraints.Billing.City = order.Billing.City;
            constraints.Billing.Company = order.Billing.Company;
            constraints.Billing.Country = order.Billing.Country;
            constraints.Billing.Fax = order.Billing.Fax;
            constraints.Billing.FirstName = order.Billing.FirstName;
            constraints.Billing.LastName = order.Billing.LastName;
            constraints.Billing.Phone = order.Billing.Phone;
            constraints.Billing.State = order.Billing.State;
            constraints.Billing.Zip = order.Billing.Zip;

            constraints.Shipping = new ShippingAddressConstraints();
            constraints.Shipping.Address1 = order.Shipping.Address1;
            constraints.Shipping.Address2 = order.Shipping.Address2;
            constraints.Shipping.City = order.Shipping.City;
            constraints.Shipping.Company = order.Shipping.Company;
            constraints.Shipping.Country = order.Shipping.Country;
            constraints.Shipping.Fax = order.Shipping.Fax;
            constraints.Shipping.FirstName = order.Shipping.FirstName;
            constraints.Shipping.LastName = order.Shipping.LastName;
            constraints.Shipping.Phone = order.Shipping.Phone;
            constraints.Shipping.State = order.Shipping.State;
            constraints.Shipping.Zip = order.Shipping.Zip;
            constraints.Shipping.IsResidential = order.Shipping.Residential;

            foreach (OrderItem orderItem in order.OrderItems)
            {
                OrderItemConstraints subconstraint = new OrderItemConstraints();
                subconstraint.DownloadCount = orderItem.DownloadCount;
                subconstraint.GiftRegistryItemID = orderItem.GiftRegistryItemID;
                subconstraint.Model = orderItem.Model;
                subconstraint.Name = orderItem.Name;
                subconstraint.OrderID = orderItem.OrderID;
                subconstraint.OrderItemID = orderItem.OrderItemID;
                subconstraint.ProductID = orderItem.ProductID;
                subconstraint.Quantity = orderItem.Quantity;
                subconstraint.RecurringID = orderItem.RecurringID;
                subconstraint.Sku = orderItem.Sku;
                subconstraint.UnitPrice = orderItem.UnitPrice;
                subconstraint.UploadFile = orderItem.UploadFile;

                if (orderItem.OrderItemRecurring != null)
                {
                    subconstraint.OrderItemRecurring = new OrderItemRecurringConstraints();
                    subconstraint.OrderItemRecurring.OrderItemID = orderItem.OrderItemRecurring.OrderItemID;
                    subconstraint.OrderItemRecurring.RecurringPeriod = orderItem.OrderItemRecurring.RecurringPeriod;
                    subconstraint.OrderItemRecurring.RecurringReceiptID = orderItem.OrderItemRecurring.RecurringReceiptID;
                    subconstraint.OrderItemRecurring.RecurringStatus = orderItem.OrderItemRecurring.RecurringStatus;
                    subconstraint.OrderItemRecurring.UpdateTime = orderItem.OrderItemRecurring.UpdateTime;
                }

                if (constraints.OrderItems == null)
                    constraints.OrderItems = new List<OrderItemConstraints>();
                constraints.OrderItems.Add( subconstraint );
            }

            return constraints;
        }

        private Order SaveOrder( Order order, OrderConstraints constraints)
        {
            if (constraints.AvsAddrStatus != null) order.AvsAddrStatus = constraints.AvsAddrStatus;
            if (constraints.AvsZipStatus != null) order.AvsZipStatus = constraints.AvsZipStatus;
            if (constraints.BaseCurrencyCode != null) order.BaseCurrencyCode = constraints.BaseCurrencyCode;          
            order.Cancelled = constraints.Cancelled;
            order.ContainsRecurring = constraints.ContainsRecurring;
            order.CouponDiscount = constraints.CouponDiscount;
            if (constraints.CouponID != null) order.CouponID = constraints.CouponID;
            if (constraints.CustomerComments != null) order.CustomerComments = constraints.CustomerComments;
            if (constraints.CustomerID != null) order.CustomerID = constraints.CustomerID;
            if (constraints.CvvStatus != null) order.CvvStatus = constraints.CvvStatus;
            if (constraints.Email != null) order.Email = constraints.Email;
            if (constraints.GatewayOrderID != null) order.GatewayOrderID = constraints.GatewayOrderID;
            if (constraints.GatewayPaymentStatus != null) order.GatewayPaymentStatus = constraints.GatewayPaymentStatus;
            order.GiftCertificate = constraints.GiftCertificate;
            if (constraints.GiftCertificateCode != null) order.GiftCertificateCode = constraints.GiftCertificateCode;
            if (constraints.GiftRegistryID != null) order.GiftRegistryID = constraints.GiftRegistryID;
            order.HandlingFee = constraints.HandlingFee;
            if (constraints.InvoiceNotes != null) order.InvoiceNotes = constraints.InvoiceNotes;
            if (constraints.IPAddress != null) order.IPAddress = constraints.IPAddress;
            order.IsQBExported = constraints.IsQBExported;
            if (constraints.OrderDate != null) order.OrderDate = constraints.OrderDate;
            if (constraints.ParentID != null) order.ParentID = constraints.ParentID;
            order.PaymentComplete = constraints.PaymentComplete;
            if (constraints.PaymentMethod != null) order.PaymentMethod = constraints.PaymentMethod;
            if (constraints.PaymentToken != null) order.PaymentToken = constraints.PaymentToken;
            order.Processed = constraints.Processed;
            if (constraints.QbPostDate != null) order.QbPostDate = constraints.QbPostDate;
            if (constraints.QbReferenceCode != null) order.QbReferenceCode = constraints.QbReferenceCode;
            if (constraints.QBTxnID != null) order.QBTxnID = constraints.QBTxnID;
            if (constraints.ShippedDate != null) order.ShippedDate = constraints.ShippedDate;
            order.ShippingCost = constraints.ShippingCost;
            if (constraints.ShippingID != null) order.ShippingID = constraints.ShippingID;
            if (constraints.ShippingMethod != null) order.ShippingMethod = constraints.ShippingMethod;
            order.ShowShippingAddress = constraints.ShowShippingAddress;
            if (constraints.Status != null) order.Status = constraints.Status;
            if (constraints.StoreID != null) order.StoreID = constraints.StoreID;
            order.Subtotal = constraints.Subtotal;
            order.Tax = constraints.Tax;
            if (constraints.TrackingMethod != null) order.TrackingMethod = constraints.TrackingMethod;
            if (constraints.TrackingNumber != null) order.TrackingNumber = constraints.TrackingNumber;
            order.UserConversionRate = constraints.UserConversionRate;
            if (constraints.UserCurrencyCode != null) order.UserCurrencyCode = constraints.UserCurrencyCode;
            if (constraints.UserName != null) order.UserName = constraints.UserName;
            if (constraints.PONumber != null) order.PONumber = constraints.PONumber;
            order.IsTaxExempt = order.IsTaxExempt;
            if (order.IsTaxExempt)
            {
                if (constraints.TaxExemptID != null) order.TaxExemptID = constraints.TaxExemptID;
                if (constraints.TaxExemptCountry != null) order.TaxExemptCountry = constraints.TaxExemptCountry;
                if (constraints.TaxExemptState != null) order.TaxExemptState = constraints.TaxExemptState;
            }

            order.RedeemPoint = constraints.RedeemPoint;
            order.RedeemPrice = constraints.RedeemPrice;
            order.PointEarned = constraints.PointEarned;
            order.IsPointApplied = constraints.IsPointApplied;
            order.IsSubscriptionApplied = constraints.IsSubscriptionApplied;

            if (constraints.Billing != null)
            {
                string firstname = String.Empty;
                string lastname = String.Empty;
                string company = String.Empty;
                string address1 = String.Empty;
                string address2 = String.Empty;
                string city = String.Empty;
                string state = String.Empty;
                string zip = String.Empty;
                string country = String.Empty;
                string phone = String.Empty;
                string fax = String.Empty;

                if (constraints.Billing.Address1 != null) address1 = constraints.Billing.Address1;
                if (constraints.Billing.Address2 != null) address2 = constraints.Billing.Address2;
                if (constraints.Billing.City != null) city = constraints.Billing.City;
                if (constraints.Billing.Company != null) company = constraints.Billing.Company;
                if (constraints.Billing.Country != null) company = constraints.Billing.Country;
                if (constraints.Billing.Fax != null) fax = constraints.Billing.Fax;
                if (constraints.Billing.Phone != null) phone = constraints.Billing.Phone;
                if (constraints.Billing.State != null) state = constraints.Billing.State;
                if (constraints.Billing.Zip != null) zip = constraints.Billing.Zip;
                if (constraints.Billing.FirstName != null) firstname = constraints.Billing.FirstName;
                if (constraints.Billing.LastName != null) lastname = constraints.Billing.LastName;
                Address address = new Vevo.Base.Domain.Address( firstname, lastname, company, address1, address2, city, state, zip, country, phone, fax );
                order.Billing = address;
            }

            if (constraints.Shipping != null)
            {
                string firstname = String.Empty;
                string lastname = String.Empty;
                string company = String.Empty;
                string address1 = String.Empty;
                string address2 = String.Empty;
                string city = String.Empty;
                string state = String.Empty;
                string zip = String.Empty;
                string country = String.Empty;
                string phone = String.Empty;
                string fax = String.Empty;

                if (constraints.Shipping.Address1 != null) address1 = constraints.Shipping.Address1;
                if (constraints.Shipping.Address2 != null) address2 = constraints.Shipping.Address2;
                if (constraints.Shipping.City != null) city = constraints.Shipping.City;
                if (constraints.Shipping.Company != null) company = constraints.Shipping.Company;
                if (constraints.Shipping.Country != null) company = constraints.Shipping.Country;
                if (constraints.Shipping.Fax != null) fax = constraints.Shipping.Fax;
                if (constraints.Shipping.Phone != null) phone = constraints.Shipping.Phone;
                if (constraints.Shipping.State != null) state = constraints.Shipping.State;
                if (constraints.Shipping.Zip != null) zip = constraints.Shipping.Zip;
                if (constraints.Shipping.FirstName != null) firstname = constraints.Shipping.FirstName;
                if (constraints.Shipping.LastName != null) lastname = constraints.Shipping.LastName;
                Address address = new Vevo.Base.Domain.Address( firstname, lastname, company, address1, address2, city, state, zip, country, phone, fax );
                ShippingAddress shippingaddress = new Vevo.Base.Domain.ShippingAddress( address, constraints.Shipping.IsResidential );
                order.Shipping = shippingaddress;
            }

            if (constraints.OrderItems != null)
            {
                order.OrderItems.Clear();
                foreach (OrderItemConstraints orderItemConstraints in constraints.OrderItems)
                {
                    OrderItem orderItem = new OrderItem();

                    orderItem.DownloadCount = orderItemConstraints.DownloadCount;
                    orderItem.GiftRegistryItemID = orderItemConstraints.GiftRegistryItemID;
                    orderItem.Model = orderItemConstraints.Model;
                    orderItem.Name = orderItemConstraints.Name;
                    orderItem.OrderID = orderItemConstraints.OrderID;
                    orderItem.OrderItemID = orderItemConstraints.OrderItemID;
                    orderItem.ProductID = orderItemConstraints.ProductID;
                    orderItem.Quantity = orderItemConstraints.Quantity;
                    orderItem.RecurringID = orderItemConstraints.RecurringID;
                    orderItem.Sku = orderItemConstraints.Sku;
                    orderItem.UnitPrice = orderItemConstraints.UnitPrice;
                    orderItem.UploadFile = orderItemConstraints.UploadFile;

                    if (orderItemConstraints.OrderItemRecurring != null)
                    {
                        orderItem.OrderItemRecurring = new OrderItemRecurring();
                        orderItem.OrderItemRecurring.OrderItemID = orderItemConstraints.OrderItemRecurring.OrderItemID;
                        orderItem.OrderItemRecurring.RecurringPeriod = orderItemConstraints.OrderItemRecurring.RecurringPeriod;
                        orderItem.OrderItemRecurring.RecurringReceiptID = orderItemConstraints.OrderItemRecurring.RecurringReceiptID;
                        orderItem.OrderItemRecurring.RecurringStatus = orderItemConstraints.OrderItemRecurring.RecurringStatus;
                        orderItem.OrderItemRecurring.UpdateTime = orderItemConstraints.OrderItemRecurring.UpdateTime;
                    }
                    order.OrderItems.Add( orderItem );
                }
            }

            return DataAccessContext.OrderRepository.Save( order );
        }

        public OrderListResponseMessage ListOrders( OrderListRequestMessage ListMessage )
        {
            EBayOrderUpdater ebayOrderUpdater = new EBayOrderUpdater();
            ebayOrderUpdater.CreateUpdateOrdersFromEBayOrder();

            OrderListResponseMessage responseMessage = new OrderListResponseMessage();
            if (WcfSessionPool.GetSession( ListMessage.Ticket ) == null)
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }

            try
            {
                int start = ListMessage.SearchFilter.StartIndex;
                int end = ListMessage.SearchFilter.EndIndex;
                int count;
                SearchFilter searchFilter = SearchFilterUtilities.ConvertConstraintToSearchFilter(
                                                ListMessage.SearchFilter );
                OrderCreateExtraFilter extraFilter = new OrderCreateExtraFilter();
                IList<Order> orderList = DataAccessContext.OrderRepository.SearchOrder(
                    "OrderID",
                    searchFilter,
                    extraFilter,
                    ListMessage.StoreID,
                    start,
                    end,
                    out count );

                responseMessage.Orders = new List<OrderConstraints>();
                foreach (Order order in orderList)
                {
                    responseMessage.Orders.Add( ConvertOrderToConstraints( order ) );
                }
            }
            catch (Exception e)
            {
                WebUtilities.LogError( e );
                responseMessage.ErrorMessage = "Error: Cannot get orders from repository";
            }

            return responseMessage;
        }

        public ResponseMessage AddOrders( OrderSaveRequestMessage AddMessage )
        {
            ResponseMessage responseMessage = new ResponseMessage();
            if (WcfSessionPool.GetSession( AddMessage.Ticket ) == null)
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }

            string errorAdd = String.Empty;
            int count = 0;
            foreach (OrderConstraints orderConstraints in AddMessage.Orders)
            {
                count += 1;
                try
                {
                    Order order_result = SaveOrder( new Order(), orderConstraints);
                    if (order_result == null)
                    {
                        errorAdd += count.ToString() + ",";
                    }
                }
                catch (Exception e)
                {
                    WebUtilities.LogError( e );
                    errorAdd += count.ToString() + ",";
                }
            }
            if (errorAdd.Length > 0)
            {
                errorAdd = errorAdd.Substring( 0, errorAdd.Length - 1 );
                responseMessage.ErrorMessage += "Error: Cannot add order(s) at :" + errorAdd;
            }
            return responseMessage;
        }

        public ResponseMessage UpdateOrders( OrderSaveRequestMessage UpdateMessage )
        {
            ResponseMessage responseMessage = new ResponseMessage();
            if (WcfSessionPool.GetSession( UpdateMessage.Ticket ) == null)
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }

            string errorUpdate = String.Empty;
            int count = 0;
            foreach (OrderConstraints orderConstraints in UpdateMessage.Orders)
            {
                count += 1;
                try
                {
                    Order order = DataAccessContext.OrderRepository.GetOne( orderConstraints.OrderID );
                    Order order_result = SaveOrder( order, orderConstraints );
                    if (order_result == null)
                    {
                        errorUpdate += count.ToString() + ",";
                    }
                }
                catch (Exception e)
                {
                    WebUtilities.LogError( e );
                    errorUpdate += count.ToString() + ",";
                }
            }
            if (errorUpdate.Length > 0)
            {
                errorUpdate = errorUpdate.Substring( 0, errorUpdate.Length - 1 );
                responseMessage.ErrorMessage += "Error: Cannot update order(s) at :" + errorUpdate;
            }
            return responseMessage;
        }

        public ResponseMessage DeleteOrders( OrderDeleteRequestMessage DeleteMessage )
        {
            ResponseMessage responseMessage = new ResponseMessage();
            if (WcfSessionPool.GetSession( DeleteMessage.Ticket ) == null)
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }

            string errorDelete = String.Empty;

            foreach (string orderID in DeleteMessage.OrderID)
            {
                try
                {
                    DataAccessContext.OrderRepository.Delete( orderID );
                    DataAccessContextDeluxe.CustomerRewardPointRepository.UpdateDeletedCustomerRewardPointDetails( orderID );
                }
                catch (Exception e)
                {
                    WebUtilities.LogError( e );
                    errorDelete += orderID + ",";
                }
            }

            if (errorDelete.Length > 0)
            {
                errorDelete = errorDelete.Substring( 0, errorDelete.Length - 1 );
                responseMessage.ErrorMessage += "Error: Cannot delete orderID(s) :" + errorDelete;
            }
            return responseMessage;
        }
    }
}
