using System;
using System.Data;
using System.Configuration;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.IO;
using System.Globalization;
using System.Threading;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.Domain;
using Vevo.WebAppLib;
using Vevo.WebUI;


namespace Vevo
{
    /// <summary>
    /// Summary description for CultureUtilities
    /// </summary>
    public static class CultureUtilities
    {
        private static string _defaultWebsiteLanguage;
        private static string _defaultID;
        private static string _languageKeywordBaseCulture;
        private static string _urlCultureName;
        private static CultureInfo _usCultureInfo;

        private static IList<Culture> _cultureList;


        private static HttpSessionState Session
        {
            get
            {
                return HttpContext.Current.Session;
            }
        }

        private static IList<Culture> CultureList
        {
            get
            {
                if (_cultureList == null)
                {
                    _cultureList = DataAccessContext.CultureRepository.GetAll();
                }

                return _cultureList;
            }
        }


        private static void LoadCultureTable()
        {
            _cultureList = null;
        }

        private static bool IsDefaultWebsiteLanguageChanged()
        {
            return _defaultWebsiteLanguage != DataAccessContext.Configurations.GetValueNoThrow( "DefaultWebsiteLanguage" );
        }

        private static bool IsLanguageKeywordBaseCultureChanged()
        {
            return _languageKeywordBaseCulture != DataAccessContext.Configurations.GetValueNoThrow( "LanguageKeywordBaseCulture" );
        }

        private static bool IsUrlCultureNameChanged()
        {
            return _urlCultureName != DataAccessContext.Configurations.GetValueNoThrow( "UrlCultureName" );
        }

        private static bool IsCultureNameExisted( string name )
        {
            bool result = false;
            for (int i = 0; i < CultureList.Count; i++)
            {
                if (CultureList[i].Name == name)
                    result = true;
            }
            return result;
        }

        private static string GetCultureIDByName( string name )
        {
            string cultureID = "0";
            for (int i = 0; i < CultureList.Count; i++)
            {
                if (CultureList[i].Name == name)
                    cultureID = CultureList[i].CultureID;
            }
            return cultureID;
        }


        public static string DefaultCultureID
        {
            get
            {
                if (IsDefaultWebsiteLanguageChanged())
                {
                    LoadCultureTable();

                    if (IsCultureNameExisted( DataAccessContext.Configurations.GetValueNoThrow( "DefaultWebsiteLanguage" ) ))
                    {
                        _defaultWebsiteLanguage = DataAccessContext.Configurations.GetValueNoThrow( "DefaultWebsiteLanguage" );
                        _defaultID = GetCultureIDByName( DataAccessContext.Configurations.GetValueNoThrow( "DefaultWebsiteLanguage" ) );
                    }
                    else if (CultureList.Count > 0)
                    {
                        _defaultWebsiteLanguage = CultureList[0].Name;
                        _defaultID = CultureList[0].CultureID;
                    }
                    else
                    {
                        _defaultWebsiteLanguage = String.Empty;
                        _defaultID = "0";
                    }
                }

                return _defaultID;
            }
        }

        public static string StoreCultureID
        {
            get
            {
                //if (Session["StoreCultureID"] == null)
                //    Session["StoreCultureID"] = DefaultCultureID;

                //return Session["StoreCultureID"].ToString();

                return StoreContext.StoreCultureID.ToString();
            }
            set
            {
                //Session["StoreCultureID"] = value;

                StoreContext.StoreCultureID = value;
            }
        }

        public static string BaseCultureID
        {
            get
            {
                if (Session["BaseCultureID"] == null ||
                    IsLanguageKeywordBaseCultureChanged())
                {
                    _languageKeywordBaseCulture = DataAccessContext.Configurations.GetValueNoThrow( "LanguageKeywordBaseCulture" );

                    Session["BaseCultureID"] = DataAccessContext.CultureRepository.GetIDByName(
                        DataAccessContext.Configurations.GetValueNoThrow( "LanguageKeywordBaseCulture" ) );
                }
                return Session["BaseCultureID"].ToString();
            }
        }

        public static string UrlCultureID
        {
            get
            {
                if (SystemConfig.ContainsKey( "UrlCultureName" ))
                {
                    if (Session == null)
                        return DataAccessContext.CultureRepository.GetIDByName( DataAccessContext.Configurations.GetValue( "UrlCultureName" ) );

                    if (Session["UrlCultureID"] == null ||
                        IsUrlCultureNameChanged())
                    {
                        Session["UrlCultureID"] = DataAccessContext.CultureRepository.GetIDByName(
                            DataAccessContext.Configurations.GetValue( "UrlCultureName" ) );

                        _urlCultureName = DataAccessContext.Configurations.GetValue( "UrlCultureName" );
                    }

                    return Session["UrlCultureID"].ToString();
                }
                else
                {
                    return CultureUtilities.DefaultCultureID;
                }
            }
        }

        public static CultureInfo USCultureInfo
        {
            get
            {
                if (_usCultureInfo == null)
                    _usCultureInfo = new CultureInfo( SystemConst.USCultureName );

                return _usCultureInfo;
            }
        }

        public static DataView GetCultureList()
        {
            DataView cultureList = (DataView) HttpContext.Current.Cache["CultureListCache"];
            if (cultureList == null)
            {
                DataSet ds = new DataSet();
                string pathFile = HttpContext.Current.Server.MapPath( "~/CultureList.xml" );
                if (File.Exists( pathFile ))
                    ds.ReadXml( pathFile );
                else
                    throw new Exception( "File Not Found : " + pathFile );

                cultureList = new DataView( ds.Tables[0] );
                cultureList.Sort = "CultureName";

                HttpContext.Current.Cache["CultureListCache"] = cultureList;
            }
            return cultureList;
        }

    }
}
