using System;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Shared.WebUI;

namespace Vevo.WebUI.Products
{
    /// <summary>
    /// Summary description for BaseProductDetails
    /// </summary>
    public class BaseProductDetails : BaseProductUserControl
    {
        #region Constructor

        public BaseProductDetails()
        {
            this.Load += new EventHandler( this.Page_Load );
        }

        #endregion

        #region Private

        private Product _product;

        #endregion

        #region Protected

        protected void Page_Load( object sender, EventArgs e )
        {
            if (!IsPostBack)
            {
                DataAccessContext.ProductRepository.IncreaseViewCount( _product.ProductID );

                if (ConvertUtilities.ToBoolean( DataAccessContext.Configurations.GetValue( "FBLikeButton" ) ))
                {
                    /*
                        og:title - The title of the entity.
                        og:type - The type of entity. You must select a type from the list of Open Graph types. http://developers.facebook.com/docs/opengraphprotocol/#types
                        og:image - The URL to an image that represents the entity. Images must be at least 50 pixels by 50 pixels. Square images work best, but you are allowed to use images up to three times as wide as they are tall.
                        og:url - The canonical, permanent URL of the page representing the entity. When you use Open Graph tags, the Like button posts a link to the og:url instead of the URL in the Like button code.
                        og:site_name - A human-readable name for your site, e.g., "IMDb".
                        fb:admins or fb:app_id - A comma-separated list of either the Facebook IDs of page administrators or a Facebook Platform application ID. At a minimum, include only your own Facebook ID.
                        facebook debug tool: http://developers.facebook.com/tools/debug
                     */

                    HtmlMeta titleTag = new HtmlMeta();
                    titleTag.Attributes.Add( "property", "og:title" );
                    titleTag.Attributes.Add( "content", _product.Name );

                    HtmlMeta typeTag = new HtmlMeta();
                    typeTag.Attributes.Add( "property", "og:type" );
                    typeTag.Attributes.Add( "content", "product" );

                    HtmlMeta imageTag = new HtmlMeta();
                    imageTag.Attributes.Add( "property", "og:image" );
                    imageTag.Attributes.Add( "content", UrlPath.StorefrontUrl + _product.GetPrimaryProductImage().RegularImage );

                    HtmlMeta urlTag = new HtmlMeta();
                    urlTag.Attributes.Add( "property", "og:url" );
                    urlTag.Attributes.Add( "content", UrlPath.GetDisplayedUrl() );

                    HtmlMeta site_nameTag = new HtmlMeta();
                    site_nameTag.Attributes.Add( "property", "og:site_name" );
                    site_nameTag.Attributes.Add( "content", DataAccessContext.Configurations.GetValue( StoreContext.Culture.CultureID, "SiteName", StoreContext.CurrentStore ) );

                    Page.Header.Controls.Add( titleTag );
                    Page.Header.Controls.Add( typeTag );
                    Page.Header.Controls.Add( imageTag );
                    Page.Header.Controls.Add( urlTag );
                    Page.Header.Controls.Add( site_nameTag );
                }
            }
        }

        public Product CurrentProduct
        {
            get
            {
                return _product;
            }
            set
            {
                _product = value;
            }
        }

        protected string CurrentProductImageID
        {
            get
            {
                if (ViewState["CurrentProductImageID"] == null)
                    ViewState["CurrentProductImageID"] = _product.GetPrimaryProductImage().ProductImageID;
                return ViewState["CurrentProductImageID"].ToString();
            }
            set
            {
                ViewState["CurrentProductImageID"] = value;
            }
        }

        protected bool VisibleQuantity( int sumStock, bool useInventory )
        {
            if (!IsAuthorizedToViewPrice())
                return false;

            if (!CatalogUtilities.IsCatalogMode() && CatalogUtilities.IsOutOfStock( sumStock, useInventory ))
                return false;
            else
                return true;
        }

        protected decimal GetPrice( object container )
        {
            object dataItem = ((IDataItemContainer) container).DataItem;
            Product product = (Product) dataItem;

            return product.GetDisplayedPrice( StoreContext.WholesaleStatus );
        }

        protected string ImageUrl( string imageName )
        {
            imageName = "~/" + imageName;
            return imageName;
        }

        protected string SmallImage
        {
            get
            {
                if (ViewState["smallImage"] == null)
                    ViewState["smallImage"] = _product.GetPrimaryProductImage().RegularImage;
                return ViewState["smallImage"].ToString();
            }

            set
            {
                ViewState["smallImage"] = value;
            }
        }

        protected string FullImage
        {
            get
            {
                if (ViewState["FullImage"] == null)
                    ViewState["FullImage"] = _product.GetPrimaryProductImage().LargeImage;
                return ViewState["FullImage"].ToString();
            }

            set
            {
                ViewState["FullImage"] = value;
            }
        }

        #endregion

        #region Public Properties

        public string DiscountGroupID
        {
            get
            {
                if (ViewState["DiscountGroupID"] == null)
                    return "0";
                else
                    return (string) ViewState["DiscountGroupID"];
            }
            set
            {
                ViewState["DiscountGroupID"] = value;
            }
        }
        #endregion

    }
}
