using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.Web;
using System.Web.Security;
using Vevo.Domain;
using Vevo.Domain.Discounts;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo;
using Vevo.WebAppLib;
using Vevo.WebUI;
using Vevo.Shared.Utilities;

namespace Vevo
{
    /// <summary>
    /// Summary description for CatalogUtilities
    /// </summary>
    public static class CatalogUtilities
    {
        private static string GetItemsPerPageSession( string configName )
        {
            if (HttpContext.Current.Session[configName + "_ItemsPerPage"] == null)
            {
                string[] values = DataAccessContext.Configurations.GetValueList( configName );
                return values[0];
            }
            else
            {
                return (string) HttpContext.Current.Session[configName + "_ItemsPerPage"];
            }
        }

        private static void SetItemsPerPageSession( string configName, string value )
        {
            HttpContext.Current.Session[configName + "_ItemsPerPage"] = value;
        }


        public static bool IsRetailMode
        {
            get
            {
                return DataAccessContext.Configurations.GetBoolValue( "RetailPriceMode" );
            }
        }

        public static string ProductItemsPerPage
        {
            get
            {
                return GetItemsPerPageSession( "ProductItemsPerPage" );
            }
            set
            {
                SetItemsPerPageSession( "ProductItemsPerPage", value );
            }
        }

        public static string ProductListView
        {
            get
            {
                return (string) HttpContext.Current.Session["ProductListView"];
            }
            set
            {
                HttpContext.Current.Session["ProductListView"] = value;
            }
        }
        public static string CategoryItemsPerPage
        {
            get
            {
                return GetItemsPerPageSession( "CategoryItemsPerPage" );
            }
            set
            {
                SetItemsPerPageSession( "CategoryItemsPerPage", value );
            }
        }

        public static string DepartmentItemsPerPage
        {
            get
            {
                return GetItemsPerPageSession( "DepartmentItemsPerPage" );
            }
            set
            {
                SetItemsPerPageSession( "DepartmentItemsPerPage", value );
            }
        }

        public static string CatalogSortField
        {
            get
            {
                return (string) HttpContext.Current.Session["CatalogSortField"];
            }
            set
            {
                HttpContext.Current.Session["CatalogSortField"] = value;
            }
        }

        public static bool IsOutOfStock( object currentStock, object useInventory )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseStockControl" ) && Convert.ToBoolean( useInventory ))
            {
                if (Convert.ToInt32( currentStock ) <= DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" ))
                    return true;
                else
                    return false;
            }
            else
                return false;

        }

        public static bool IsCatalogMode()
        {
            if (DataAccessContext.Configurations.GetBoolValue( "IsCatalogMode" ))
                return true;
            else
                return false;
        }

        public static bool GetQuantityDiscountByProductID(
            object productDiscountGroupID,
            object categoryIDList )
        {
            string discountGroupID = "0";
            IList<String> categoryIDs = (IList<String>) categoryIDList;
            if (!String.IsNullOrEmpty( productDiscountGroupID.ToString() ) && productDiscountGroupID.ToString() != "0")
                discountGroupID = productDiscountGroupID.ToString();
            else if (categoryIDs.Count > 0)
            {
                discountGroupID = DataAccessContext.CategoryRepository.GetOne( StoreContext.Culture, categoryIDs[0].ToString() ).DiscountGroupID.ToString();// categoryDiscountGroupID.ToString();
            }

            return DiscountGroup.HasDiscountRule( discountGroupID );
        }

    }
}
