﻿using System;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Blogs;
using Vevo.Domain.DataInterfaces;
using Vevo.WebUI;
using Vevo.WebUI.Products;
using Vevo.Base.Domain;

public partial class Layouts_BlogLists_BlogListRowStyle : BaseProductListControl
{
    private const string _sortBy = "CreateDate DESC";
    private int _blogItemPerPage = DataAccessContext.Configurations.GetIntValue( "BlogListItemsPerPage", StoreContext.CurrentStore );

    private string ItemPerPage
    {
        get
        {
            if (ViewState["ItemPerPage"] == null)
                ViewState["ItemPerPage"] = uxItemsPerPageControl.DefaultValue;

            return (string) ViewState["ItemPerPage"];
        }
        set
        {
            ViewState["ItemPerPage"] = value;
        }
    }

    private string CurrentBlogListName
    {
        get
        {
            if (String.IsNullOrEmpty( Request.QueryString["BlogListName"] ))
            {
                return String.Empty;
            }
            else
            {
                return Request.QueryString["BlogListName"];
            }
        }
    }

    private string CurrentSearch
    {
        get
        {
            if (Request.QueryString["Search"] == null)
                return String.Empty;
            else
                return Request.QueryString["Search"];
        }
    }

    private IList<Blog> GetBlogList( int itemsPerPage, string sortBy, out int totalItem )
    {
        if (!String.IsNullOrEmpty( CurrentBlogListName ))
        {
            string[] date = CurrentBlogListName.Split( '_' );
            return DataAccessContext.BlogRepository.GetBlogListByDate(
                date[0],
                date[1],
                (uxPagingControl.CurrentPage - 1) * itemsPerPage,
                (uxPagingControl.CurrentPage * itemsPerPage) - 1,
                out totalItem,
                StoreContext.CurrentStore.StoreID );
        }
        else if (!String.IsNullOrEmpty( CurrentSearch ))
        {
            return DataAccessContext.BlogRepository.QuickSearch(
                CurrentSearch,
                new string[] { "BlogTitle", "ShortContent", "BlogContent", "Tags" },
                (uxPagingControl.CurrentPage - 1) * itemsPerPage,
                (uxPagingControl.CurrentPage * itemsPerPage) - 1,
                out totalItem,
                StoreContext.CurrentStore.StoreID );
        }
        else
        {
            return DataAccessContext.BlogRepository.SearchBlog(
                sortBy,
                new SearchFilter(),
                (uxPagingControl.CurrentPage - 1) * itemsPerPage,
                (uxPagingControl.CurrentPage * itemsPerPage) - 1,
                out totalItem,
                StoreContext.CurrentStore.StoreID,
                false,
                BoolFilter.ShowTrue );
        }
    }

    private void PopulateProductControls()
    {
        if (this.Visible)
            Refresh();
    }

    private void BlogList_StoreCultureChanged( object sender, CultureEventArgs e )
    {
        Refresh();
    }

    private void BlogList_StoreCurrencyChanged( object sender, CurrencyEventArgs e )
    {
        Refresh();
    }

    private void RegisterStoreEvents()
    {
        GetStorefrontEvents().StoreCultureChanged +=
            new StorefrontEvents.CultureEventHandler( BlogList_StoreCultureChanged );
        GetStorefrontEvents().StoreCurrencyChanged +=
            new StorefrontEvents.CurrencyEventHandler( BlogList_StoreCurrencyChanged );
    }

    private ScriptManager GetScriptManager()
    {
        return (ScriptManager) Page.Master.FindControl( "uxScriptManager" );
    }

    private void AddHistoryPoint()
    {
        GetScriptManager().AddHistoryPoint( "page", uxPagingControl.CurrentPage.ToString() );
        GetScriptManager().AddHistoryPoint( "itemPerPage", uxItemsPerPageControl.SelectedValue );
    }

    protected void uxItemsPerPageControl_BubbleEvent( object sender, EventArgs e )
    {
        ItemPerPage = uxItemsPerPageControl.SelectedValue;
        uxPagingControl.CurrentPage = 1;
        AddHistoryPoint();
        Refresh();
    }

    protected void uxPagingControl_BubbleEvent( object sender, EventArgs e )
    {
        AddHistoryPoint();
        Refresh();
    }

    protected void ScriptManager_Navigate( object sender, HistoryEventArgs e )
    {
        string args;

        if (!string.IsNullOrEmpty( e.State["itemPerPage"] ))
        {
            ItemPerPage = e.State["itemPerPage"].ToString();
        }
        else
        {
            ItemPerPage = uxItemsPerPageControl.DefaultValue;
        }

        int totalItems = 50;
        int selectedValue;
        selectedValue = Convert.ToInt32( uxItemsPerPageControl.SelectedValue );

        uxPagingControl.NumberOfPages = (int) System.Math.Ceiling( (double) totalItems / selectedValue );

        if (!string.IsNullOrEmpty( e.State["page"] ))
        {
            args = e.State["page"];
            uxPagingControl.CurrentPage = int.Parse( args );
        }
        else
        {
            uxPagingControl.CurrentPage = 1;
        }

        Refresh();
    }

    protected void Page_Load( object sender, EventArgs e )
    {
        RegisterStoreEvents();

        uxPagingControl.BubbleEvent += new EventHandler( uxPagingControl_BubbleEvent );
        uxItemsPerPageControl.BubbleEvent += new EventHandler( uxItemsPerPageControl_BubbleEvent );
        GetScriptManager().Navigate += new EventHandler<HistoryEventArgs>( ScriptManager_Navigate );

        uxList.RepeatColumns = 1;
        uxPageControlTR.Visible = true;

        if (!IsPostBack)
        {
            ItemPerPage = _blogItemPerPage.ToString();
            uxBlogListNoDataDiv.Visible = false;
        }
    }

    protected void Page_PreRender( object sender, EventArgs e )
    {
        if (this.Visible)
        {
            if (ViewState["UserDefinedParameters"] != null)
            {
                PopulateProductControls();
            }
            else
            {
                uxItemsPerPageControl.SelectValue( ItemPerPage );
            }

            PopulateProductControls();
        }
    }

    protected void uxList_ItemDataBound( object sender, DataListItemEventArgs e )
    {
    }

    public void PopulateText( int totalItems )
    {
        if (totalItems > 0)
        {
            uxBlogListNoDataDiv.Visible = false;
            uxMessageLabel.Text = "";
        }
        else
        {
            uxBlogListNoDataDiv.Visible = true;
            uxPageControlTR.Visible = false;
            uxMessageLabel.Text = "<div style='text-align: center;'>[$BlogListNoResults]</div>";
        }
    }

    public void Refresh()
    {
        uxItemsPerPageControl.SelectValue( ItemPerPage );

        int totalItems = 0;
        int itemPerPage = Convert.ToInt32( uxItemsPerPageControl.SelectedValue );
        uxList.DataSource = GetBlogList( itemPerPage, _sortBy, out totalItems );
        uxList.DataBind();

        uxPagingControl.NumberOfPages = (int) System.Math.Ceiling( (double) totalItems / itemPerPage );
        PopulateText( totalItems );
    }
}
